package software.eskimo.capacitor.sockets;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ServiceInfo;
import android.os.Build;
import android.os.IBinder;
import androidx.core.app.NotificationCompat;
import android.util.Log;

public class SocketForegroundService extends Service {
    private static final String CHANNEL_ID = "socket_channel";
    private static final String CHANNEL_NAME = "Socket Service";
    private static final int NOTIFICATION_ID = 1;

    @Override
    public void onCreate() {
        super.onCreate();
        createNotificationChannel();
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        String action = intent != null ? intent.getAction() : null;
        String notificationText = intent != null ? 
            intent.getStringExtra("notificationText") : "Maintaining connection";

        if ("software.eskimo.capacitor.sockets.UPDATE_NOTIFICATION".equals(action)) {
            updateNotification(notificationText);
        } else {
            startForegroundWithNotification(notificationText);
        }

        return START_STICKY;
    }

    private void startForegroundWithNotification(String notificationText) {
        int appNameResId = getResources().getIdentifier("app_name", "string", getApplicationContext().getPackageName());
        String appName = (appNameResId != 0) ? getString(appNameResId) : "Socket Service";

        int iconResId = getResources().getIdentifier("ic_notification", "drawable", getApplicationContext().getPackageName());
        if (iconResId == 0) {
            iconResId = android.R.drawable.ic_dialog_info;
        }

        Notification notification = new NotificationCompat.Builder(this, CHANNEL_ID)
            .setContentTitle(appName)
            .setContentText(notificationText)
            .setSmallIcon(iconResId)
            .setPriority(NotificationCompat.PRIORITY_LOW)
            .build();

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            startForeground(NOTIFICATION_ID, notification, ServiceInfo.FOREGROUND_SERVICE_TYPE_CONNECTED_DEVICE);
        } else {
            startForeground(NOTIFICATION_ID, notification);
        }

        Log.d("SocketForegroundService", "Foreground service started with app name: " + appName);
    }

    private void updateNotification(String notificationText) {
        int appNameResId = getResources().getIdentifier("app_name", "string", getApplicationContext().getPackageName());
        String appName = (appNameResId != 0) ? getString(appNameResId) : "Socket Service";

        int iconResId = getResources().getIdentifier("ic_notification", "drawable", getApplicationContext().getPackageName());
        if (iconResId == 0) {
            iconResId = android.R.drawable.ic_dialog_info;
        }

        Notification notification = new NotificationCompat.Builder(this, CHANNEL_ID)
            .setContentTitle(appName)
            .setContentText(notificationText)
            .setSmallIcon(iconResId)
            .setPriority(NotificationCompat.PRIORITY_LOW)
            .build();

        NotificationManager notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
        if (notificationManager != null) {
            notificationManager.notify(NOTIFICATION_ID, notification);
        }

        Log.d("SocketForegroundService", "Notification updated with text: " + notificationText);
    }

    @Override
    public void onTaskRemoved(Intent rootIntent) {
        Log.d("SocketForegroundService", "App task removed. Stopping foreground service.");
        stopSelf();
        super.onTaskRemoved(rootIntent);
    }

    private void createNotificationChannel() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            NotificationChannel channel = new NotificationChannel(
                CHANNEL_ID,
                CHANNEL_NAME,
                NotificationManager.IMPORTANCE_LOW
            );
            channel.setDescription("Channel for socket foreground service");
            NotificationManager notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
            if (notificationManager != null) {
                notificationManager.createNotificationChannel(channel);
            }
        }
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }
}