package software.eskimo.capacitor.sockets;

import android.app.ActivityManager;
import android.content.Context;
import android.content.Intent;
import android.util.Log;
import com.getcapacitor.JSObject;
import com.getcapacitor.Plugin;
import com.getcapacitor.PluginCall;
import com.getcapacitor.annotation.CapacitorPlugin;
import com.getcapacitor.PluginMethod;

@CapacitorPlugin(name = "Sockets")
public class SocketsPlugin extends Plugin {

    private Sockets implementation = new Sockets(this);

    @PluginMethod
    public void create(PluginCall call) {
        String id = call.getString("id", java.util.UUID.randomUUID().toString());
        implementation.create(id);
        call.resolve();
    }

    @PluginMethod
    public void connect(PluginCall call) {
        String id = call.getString("id", "");
        String host = call.getString("host", "");
        int port = call.getInt("port", 0);
        boolean useTLS = call.getBoolean("useTLS", false);
        boolean acceptInvalidCertificates = call.getBoolean("acceptInvalidCertificates", false);
        String delimiter = call.getString("delimiter", "\r\n");

        startSocketForegroundService();
        
        implementation.connect(id, host, port, useTLS, acceptInvalidCertificates, delimiter);
        call.resolve();
    }

    @PluginMethod
    public void send(PluginCall call) {
        String id = call.getString("id", "");
        String message = call.getString("message", "");

        implementation.send(id, message);
        call.resolve();
    }

    @PluginMethod
    public void disconnect(PluginCall call) {
        String id = call.getString("id", "");

        implementation.disconnect(id);
        if (implementation.getActiveSocketCount() == 0) {
            stopSocketForegroundService();
        }
        call.resolve();
    }

    @PluginMethod
    public void setNotificationText(PluginCall call) {
        String notificationText = call.getString("notificationText", "Maintaining connection");
        
        Intent intent = new Intent(getContext(), SocketForegroundService.class);
        intent.setAction("software.eskimo.capacitor.sockets.UPDATE_NOTIFICATION");
        intent.putExtra("notificationText", notificationText);
        
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            getContext().startForegroundService(intent);
        } else {
            getContext().startService(intent);
        }
        
        call.resolve();
    }

    public void notifyStateListeners(String event, String id, String state) {
        JSObject data = new JSObject();
        data.put("id", id);
        data.put("state", state);
        notifyListeners(event, data);
    }

    public void notifyMessageListeners(String id, String message) {
        JSObject data = new JSObject();
        data.put("id", id);
        data.put("message", message);
        
        Log.d("SocketsPlugin", "Message to JS: " + data.toString());

        notifyListeners("message", data);
    }

    private void startSocketForegroundService() {
        if (!isServiceRunning(SocketForegroundService.class)) {
            Intent serviceIntent = new Intent(getContext(), SocketForegroundService.class);
            serviceIntent.putExtra("notificationText", "Maintaining connection");
            getContext().startForegroundService(serviceIntent);
        }
    }

    private void stopSocketForegroundService() {
        Intent serviceIntent = new Intent(getContext(), SocketForegroundService.class);
        getContext().stopService(serviceIntent);
    }

    private boolean isServiceRunning(Class<?> serviceClass) {
        ActivityManager manager = (ActivityManager) getContext().getSystemService(Context.ACTIVITY_SERVICE);
        for (ActivityManager.RunningServiceInfo service : manager.getRunningServices(Integer.MAX_VALUE)) {
            if (serviceClass.getName().equals(service.service.getClassName())) {
                return true;
            }
        }
        return false;
    }
}